/*
 * Copyright (c) 2011, Disgruntled Rats
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of the Disgruntled Rats nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

package com.disgruntledrats.item;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import android.util.Pair;


/**
 * Stores and retrieves instances of <tt>Item</tt>.  This is an abstraction
 * that makes it easier to find things that were purchased.
 */
public class Safe {

    private final Map<String, Stack<Item>> itemMap
        = new HashMap<String, Stack<Item>>();

    private static final Safe INSTANCE = new Safe();

    /**
     * Hiding the constructor because it is a singleton
     */
    private Safe() {
    }

    /**
     * Removes all the items from the safe
     */
    public void clear() {
        itemMap.clear();
    }

    /**
     * Adds the item to this safe.
     * @param item
     *          the item you want to add to the safe
     */
    public void addItem(Item item) {
        Stack<Item> itemStack;
        String itemId = item.getId();
        if (!itemMap.containsKey(itemId)) {
            itemStack = new Stack<Item>();
            itemMap.put(itemId, itemStack);
        } else {
            itemStack = itemMap.get(itemId);
        }
        itemStack.push(item);
    }

    public void addItem(Item item, int quantity) {
        for (int i = 0; i < quantity; i++) {
            addItem(item.copy());
        }
    }

    /**
     * Removes an item from the safe
     * @param itemId
     *          the id of the item to remove
     */
    public void removeItem(String itemId) {
        Stack<Item> itemStack;
        if (itemMap.containsKey(itemId)) {
            itemStack = itemMap.get(itemId);
            if (!itemStack.isEmpty()) {
                itemStack.pop();
            }
        }
    }

    /**
     * Gets the list of items from the safe with the given item id
     * @param itemId
     *          the id of the items you are interested in
     * @return
     *          the list of items, or null if none exist with the given id
     */
    public List<Item> getItems(String itemId) {
        List<Item> itemList = itemMap.get(itemId);
        if (itemList == null || itemList.size() == 0) {
            return null;
        } else {
            return itemList;
        }
    }

    /**
     * Gets the singleton instance
     * @return
     *          the instance
     */
    public static Safe getInstance() {
        return INSTANCE;
    }

    /**
     * Adds a purchase to this safe
     * @param purchase
     *          the purchase to add
     */
    public void addPurchase(Purchase purchase) {
        List<Pair<Item, Integer>> itemList = purchase.getItems();
        for (Pair<Item, Integer> pair : itemList) {
            addItem(pair.first, pair.second);
        }
    }

    /*
     * (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "Current Safe Contents: [itemMap=" + itemMap + "]";
    }
}
