/*
 * Copyright (c) 2011, Disgruntled Rats
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of the Disgruntled Rats nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

package com.disgruntledrats.item;

import java.util.Map;
import java.util.Observable;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.os.Handler;
import android.util.Log;

import com.disgruntledrats.BillingActivity;
import com.disgruntledrats.billing.android.BillingService;
import com.disgruntledrats.billing.android.BillingService.RequestPurchase;
import com.disgruntledrats.billing.android.BillingService.RestoreTransactions;
import com.disgruntledrats.billing.android.Consts;
import com.disgruntledrats.billing.android.Consts.PurchaseState;
import com.disgruntledrats.billing.android.Consts.ResponseCode;
import com.disgruntledrats.billing.android.AndroidMarketAccount;
import com.disgruntledrats.billing.android.PurchaseDatabase;
import com.disgruntledrats.billing.android.PurchaseObserver;
import com.disgruntledrats.billing.android.ResponseHandler;

/**
 * The purpose of this class is to hold a list of <tt>DisposableItem<tt> and
 * <tt>PermanentItem</tt> instances.  An application just wants to know what
 * items are for sale and have a way to purchase them.  This class does
 * exactly that.
 */
public class Store extends Observable {

    // inventory
    private Map<String, DisposableItem> disposableItemMap;
    private Map<String, PermanentItem> permanentItemMap;

    // the store cannot be used until it is initialized
    private boolean initialized = false;

    private PurchaseDatabase mPurchaseDatabase;
    private StorePurchaseObserver storePurchaseObserver;
    private final Safe SAFE = Safe.getInstance();
    private BillingService mBillingService = new BillingService();
    private BillingActivity activity;

    /**
     * The SharedPreferences key for recording whether we initialized the
     * database.  If false, then we perform a RestoreTransactions request
     * to get all the purchases for this user.
     */
    private static final String DB_INITIALIZED = "db_initialized";

    private static final Store INSTANCE = new Store();

    /**
     * Hiding the constructor because it's a singleton
     */
    private Store() {
    }

    /**
     * Initializes the store with the sets of available items
     * @param disposableItemSet
     * @param permanentItemSet
     */
    public void initialize(BillingActivity activity,
            Map<String, DisposableItem> disposableItemSet,
            Map<String, PermanentItem> permanentItemSet) {
        this.activity = activity;
        mPurchaseDatabase = new PurchaseDatabase(activity);
        storePurchaseObserver = new StorePurchaseObserver(activity,
                new Handler());
        mBillingService.setContext(activity);
        this.disposableItemMap = disposableItemSet;
        this.permanentItemMap = permanentItemSet;
        restoreItemsFromDatabase();
        restorePermanentItemsFromStore();
        initialized = true;
    }

    /**
     * Initializes this safe with instances of <tt>PermanentItem</tt> missing
     * from the store.
     * @param activity
     */
    private void restorePermanentItemsFromStore() {
      SharedPreferences prefs = activity.getPreferences(Context.MODE_PRIVATE);
      boolean initialized = prefs.getBoolean(DB_INITIALIZED, false);
      if (!initialized) {
          mBillingService.restoreTransactions();
      }
    }

    /**
     * Gets the singleton instance
     * @return
     *          the singleton
     */
    public static Store getInstance() {
        return INSTANCE;
    }

    /**
     * Checks to see if the store is open and billing is enabled.
     * @return
     *          true if the store is open and billing is enabled
     */
    public boolean isOpen() {
        return mBillingService.checkBillingSupported();
    }

    /**
     * Closes this store and all the connections
     */
    public void close() {
        mPurchaseDatabase.close();
        mBillingService.unbind();
    }

    /**
     * Purchases an item.  The item will appear in your safe asynchronously
     * if the purchase succeeds.
     * @param itemId
     *          the item id to purchase
     * @return
     *          true if the purchase request operation succeeds
     */
    public boolean purchase(String itemId) {
        if (!initialized) {
            return false;
        }
        // TODO figure out what is in the payload
        if (mBillingService.requestPurchase(itemId, null)) {
            return true;
        } else {
            System.err.println("Purchase failed of item: " + itemId);
            return false;
        }
    }

    /**
     * Gets a copy of the item from the store.  This method is only meant to
     * be used to restore already purchased items.  Not for new purchases.
     * @param itemId
     *          the item id
     * @return
     *          a copy of the item
     */
    protected Item getItemCopy(String itemId) {
        if (disposableItemMap.containsKey(itemId)) {
            return disposableItemMap.get(itemId).copy();
        } else if (permanentItemMap.containsKey(itemId)) {
            return permanentItemMap.get(itemId).copy();
        } else {
            return null;
        }
    }

    /**
     * Determines if the given item id is a disposable item.
     * @param itemId
     *          the item id
     * @return
     *          true if the item id is a disposable item
     */
    public boolean isDisposableItem(String itemId) {
        return disposableItemMap.containsKey(itemId);
    }

    /**
     * Determines if the given item is a permanent item
     * @param itemId
     *          the id of the item
     * @return
     *          true if the item id is a permanent item
     */
    public boolean isPermanentItem(String itemId) {
        return permanentItemMap.containsKey(itemId);
    }

    /**
     * Restores items from the database into memory.
     */
    public void restoreItemsFromDatabase() {
        Cursor cursor = mPurchaseDatabase.queryAllPurchasedItems();
        if (cursor == null) {
            return;
        }

        try {
            int productIdCol = cursor.getColumnIndexOrThrow(
                    PurchaseDatabase.PURCHASED_PRODUCT_ID_COL);
            while (cursor.moveToNext()) {
                String productId = cursor.getString(productIdCol);
                SAFE.addItem(getItemCopy(productId));
            }
        } finally {
            cursor.close();
        }
    }

    private class StorePurchaseObserver extends PurchaseObserver {

        public StorePurchaseObserver(Activity activity, Handler handler) {
            super(activity, handler);
            ResponseHandler.register(this);
        }

        /*
         * (non-Javadoc)
         * @see com.disgruntledrats.billing.android.PurchaseObserver#onBillingSupported(boolean)
         */
        @Override
        public void onBillingSupported(boolean supported) {
            if (Consts.DEBUG) {
                Log.d(Consts.TAG, "Store.onBillingSupported(" + supported + ")");
            }
        }

        /*
         * (non-Javadoc)
         * @see com.disgruntledrats.billing.android.PurchaseObserver#onPurchaseStateChange(com.disgruntledrats.billing.android.Consts.PurchaseState, java.lang.String, int, long, java.lang.String)
         */
        @Override
        public void onPurchaseStateChange(PurchaseState purchaseState,
                String itemId, int quantity, long purchaseTime,
                String developerPayload) {
            if (Consts.DEBUG) {
                Log.d(Consts.TAG, "Store.onPurchaseStateChange("
                        + purchaseState + ", " + itemId + ", " + quantity
                        + ", " + purchaseTime + ", " + developerPayload + ")");
            }
            switch (purchaseState) {
            case CANCELED:
                return;
            case PURCHASED:
                for (int i = 0; i < quantity; i++) {
                    Item item = getItemCopy(itemId);
                    item.setPurchaseTime(purchaseTime);
                    SAFE.addItem(item);
                }
                return;
            case REFUNDED:
                for (int i = 0; i < quantity; i++) {
                    SAFE.removeItem(itemId);
                }
                return;
            }
        }

        /*
         * (non-Javadoc)
         * @see com.disgruntledrats.billing.android.PurchaseObserver#onRequestPurchaseResponse(com.disgruntledrats.billing.android.BillingService.RequestPurchase, com.disgruntledrats.billing.android.Consts.ResponseCode)
         */
        @Override
        public void onRequestPurchaseResponse(RequestPurchase request,
                ResponseCode responseCode) {
            if (Consts.DEBUG) {
                Log.d(Consts.TAG, "Store.onRequestPurchaseResponse("
                        + request + ", " + responseCode + ")");
            }
            if (responseCode == ResponseCode.RESULT_OK) {
                if (Consts.DEBUG) {
                    Log.i(Consts.TAG, "purchase was successfully sent to server");
                }
            } else if (responseCode == ResponseCode.RESULT_USER_CANCELED) {
                if (Consts.DEBUG) {
                    Log.i(Consts.TAG, "user canceled purchase");
                }
            } else {
                if (Consts.DEBUG) {
                    Log.i(Consts.TAG, "purchase failed");
                }
            }
        }

        /*
         * (non-Javadoc)
         * @see com.disgruntledrats.billing.android.PurchaseObserver#onRestoreTransactionsResponse(com.disgruntledrats.billing.android.BillingService.RestoreTransactions, com.disgruntledrats.billing.android.Consts.ResponseCode)
         */
        @Override
        public void onRestoreTransactionsResponse(RestoreTransactions request,
                ResponseCode responseCode) {
            if (responseCode == ResponseCode.RESULT_OK) {
                if (Consts.DEBUG) {
                    Log.d(Consts.TAG, "Store.onRestoreTransactionsResponse("
                            + request + ", " + responseCode + ")");
                }
                // Update the shared preferences so that we don't perform
                // a RestoreTransactions again.
                SharedPreferences prefs = activity.getPreferences(Context.MODE_PRIVATE);
                SharedPreferences.Editor edit = prefs.edit();
                edit.putBoolean(DB_INITIALIZED, true);
                edit.commit();
            } else {
                if (Consts.DEBUG) {
                    Log.d(Consts.TAG, "RestoreTransactions error: " + responseCode);
                }
            }
        }
        
    }

    /**
     * Gets the Android Market Account for your company.
     * @return
     *          the Account for your company
     */
    public AndroidMarketAccount getAndroidMarketAccount() {
        return activity.getAndroidMarketAccount();
    }
}
